
import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import java.awt.*;
import java.util.*;
import java.applet.*;
import java.awt.event.*;
//import rmibook.util.*;


public class ChatImpl 
   extends Applet
   implements Chat, ChatApplet {

   private TextArea ta;       //the main text window
   private TextField tf;      //used to input messages
   private ChatServer cs;     //a reference to the server
   private String name = null;//The user's name
   private DrawPad dp;        //our Whiteboard app
   private Dialog nd;     //pop-up to request user name
   private ServerTalker st;   //Thread for handling message sending

   public ChatImpl() throws RemoteException {
      System.out.println("Starting up Chatter");
   }

   public void init(){
      //set the Applets layout manager
      this.setLayout(new BorderLayout());

      //create a Panel to add our button
      //interface to
      Panel p = new Panel();
      p.setLayout(new FlowLayout());
      //add our interface buttons to the Panel
      Button bList = new Button("List");
      //p.add(new Button("List"));   //list users
      p.add(bList);   //list users
      Button bClearDraw = new Button("ClearDraw");
      //p.add(new Button("ClearDraw")); //clear drawing screen
      p.add(bClearDraw); //clear drawing screen
      Button bClearText = new Button("ClearText");
      //p.add(new Button("ClearText")); //clear text screen
      p.add(bClearText); //clear text screen
      Button b = new Button("Unregister"); //disconnect from server
      b.setBackground(Color.pink); //for dramatic effect
      p.add(b);

      //manejador de eventos para el botn bList
      bList.addActionListener(
        new ActionListener() {
           public void actionPerformed(ActionEvent e) {
              getUserList();
           }
        }
      );

      //manejador de eventos para el botn bClearText
      bClearText.addActionListener(
        new ActionListener() {
           public void actionPerformed(ActionEvent e) {
              ta.setText("");
           }
        }
      );

      //manejador de eventos para el botn bClearDraw
      bClearDraw.addActionListener(
        new ActionListener() {
           public void actionPerformed(ActionEvent e) {
              dp.clearScreen();
           }
        }
      );

      //manejador de eventos para el botn b-> unregister (desconectar)
      b.addActionListener(
        new ActionListener() {
           public void actionPerformed(ActionEvent e) {
              dp.clearScreen();
           }
        }
      );

      //create our text widgets and drawing window
      ta = new TextArea(4,40); //message window
      ta.setEditable(false);
      tf = new TextField(40);  //text entry field
      dp = new DrawPad(this);  //whiteboard
      dp.setSize(400,400);

      //manejador de eventos para el campo de texto 
      tf.addActionListener(
        new ActionListener() {
           public void actionPerformed(ActionEvent e) {
             if (!st.addMessage(new Message(name,tf.getText().trim())))
               ta.append("***Server Error\n");
             else System.out.println("texto enviado");
             tf.setText(""); 
           }
        }
      );

      //add all of our widgets to the Applet
      add(dp,"East");
      add(ta,"Center");
      add(tf,"South");
      add(p,"North");

      //create a dialogue box to ask for the user name
      nd = new 
	Dialog(new Frame("Enter Name"),"Enter your name", false);
      registerChatter(); //register the Applet with the server
  }

  public void registerChatter(){
      name = nd.getName(); //get name from NameDialog
      nd.setVisible(false);//get rid of NameDialog
      nd = null;

      try {
	//export our remote methods
        UnicastRemoteObject.exportObject(this);
        System.out.println("Objeto exportado...");

	//lookup the server's remote object
        System.out.println("Buscando objeto remoto: "+ "//" + 
		java.net.InetAddress.getLocalHost().getHostAddress() +":"+
                "1200/"+ "ChatServerImpl");
	
	cs = (ChatServer)Naming.lookup("//" +
		java.net.InetAddress.getLocalHost().getHostAddress() +":"+
                "1200/"+ "ChatServerImpl");

        System.out.println("ChatServer encontrado...");
	//register our Applet with the server
        System.out.println("Registramos el applet con el nombre: "+name);
	cs.register(this,name);
        System.out.println("Applet registrado en el Servidor");

	//start a communication thread
	st = new ServerTalker(cs,name);
      } catch (RemoteException e) {
	System.out.println("Couldn't locate Registry ");
         e.printStackTrace();
      } catch (MalformedURLException e) {
	System.out.println("Bad binding URL: ");
	System.exit(0);
      } catch(NotBoundException e) {
	System.out.println("Service not bound." );
        e.printStackTrace();
	System.exit(0);
      } catch (Exception e) {
         e.printStackTrace();
      }

  }

/*
  public boolean handleEvent(Event e){
    if(e.target == tf&&e.id==Event.ACTION_EVENT) {
       if(!st.addMessage(new Message(name,tf.getText().trim())))
          ta.append("***Server Error***\n");
          tf.setText("");
          return true;
       }
      if(e.target instanceof Button){
         if(e.arg == "ClearText") {
         ta.setText("");
         } else if(e.arg == "ClearDraw") {
            dp.clearScreen();
         } else if(e.arg == "Disconnect") {
            st.addMessage(new Message("***"+name,"Logged off. Bye"));
            cs = null;
            System.exit(0);
            return true;
         } else if(e.arg == "List") {
            getUserList();
         }
      }
      return false;
  }
*/

  public void sendMessage(Line l) {
      //System.out.println("Sending line");
      if(!st.addMessage(new Message(name,l)))
	ta.append("***Server Error***\n");
  }

   public void getUserList(){
     String users[] = null;
     try {
	users = cs.listChatters();
     } catch (RemoteException e) {
	System.out.println(e);
	users = new String[1];
	users[0] = "***Error";
     }

	for(int i = 0; i < users.length; i++)
	   ta.append("***"+users[i]+"\n");
   }
   public synchronized void chatNotify(Message m) throws RemoteException {
	if(m.getMessage()!=null)
	   ta.append(m.getSender() + ": " +m.getMessage()+"\n");
	//if(m.getLine()!=null&&!m.getSender().equals(name))
	if(m.getLine()!=null)
	   dp.addLine(m.getLine());
   }

   public String getName(){
	return name;
   }

}
